import omp.core.entry as entry
import omp.core.openmp as openmp
import omp.core.threading as threading
import omp.core.ast_tools as ast_tools

# Avoid linter warnings for package shortcuts definitions.
entry
openmp
threading
ast_tools
